//
// Asynchronous read memory 
//

module async_read_mem 
#(parameter 
   WIDTH   = 16,
   L2DEPTH = 8
  )
(
   // System
   input   clk,
   
   // Write port
   input                write,
   input  [L2DEPTH-1:0] wadrs,
   input    [WIDTH-1:0] wdata,
   
   // Read port (Asynchronous!)
   input  [L2DEPTH-1:0] radrs,
   output   [WIDTH-1:0] rdata
   );

localparam DEPTH = 1<<L2DEPTH;   
   
reg [WIDTH-1:0] memory [0:DEPTH-1];

   always @(posedge clk)
   begin
      if (write)
         memory[wadrs] <= wdata;
   end
   
   assign rdata = memory[radrs];
   
endmodule
   
   
