//
// Single ported memory
// Synchronous with byte write enables
//
// Freeware September 2015, Fen Logic Ltd.
// This code is free and is delivered 'as is'.
// It comes without any warranty, to the extent permitted by applicable law.
// There are no restrictions to any use or re-use of this code
// in any form or shape. It would be nice if you keep my company name
// in the source or modified source code but even that is not
// required as I can't check it anyway.
// But the code comes with no warranties or guarantees whatsoever.
//
//
// Known issues:
//    The code casues an internal error in the ISE tool (14.7) I am using
//    I have not found a work around for that yet. 
//

module spmbe
#(parameter BYTES    = 4, // width in bytes
            L2D      = 8, // Log 2 Depth, 8 = 256 deep 
            // Simulation parameters
            CLEAR    = 0,
            FILE     = ""
)
(
   input                    clk,     // system clock 
               
   input                    enable,  // access 
   input                    write,   // write 
   input        [BYTES-1:0] bwe,     // Byte write enable 
   input          [L2D-1:0] address, // read/write address
   input      [BYTES*8-1:0] wdata,   // write data
   output reg [BYTES*8-1:0] rdata    // read data
);

localparam DEPTH = (1<<L2D);

reg [BYTES*8-1:0] memory [0:DEPTH-1];

   always @(posedge clk)
   begin : mm
      integer b;
      if (enable)
      begin
         if (write)
         begin
            for (b=0; b<BYTES; b=b+1)
               if (bwe[b])
                 memory[address][b*8 +: 8] <= wdata[b*8 +: 8];
         end
         else
            rdata <= memory[address];
      end
   end

/*
  synthesis translate_...
  synopsys  translate_...
  pragma    translate_...
 */
//synopsys translate_off
integer i;
   initial
   begin
      if (CLEAR)
      begin
         for (i=0; i<DEPTH; i=i+1)
            memory[i] = 0;
      end
      #0; // prevent simultaneous clear and file loading 
      // (Yes: it is all blocking code but still I have seen it happen)
      
      // If the filename is not zero use it to load from
      if (FILE[7:0]!=0)
         $readmemh(FILE,memory);
   end
//synopsys translate_on

endmodule